# Copyright Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from os import environ
from typing import Optional

logger = logging.getLogger("options")


DEFAULT_SERVICE_NAME = "unnamed-python-service"
DEFAULT_ENDPOINT = "http://localhost:9080/v1/trace"
DEFAULT_MAX_ATTR_LENGTH = 1200


class Options:
    endpoint: str
    service_name: str
    access_token: str
    max_attr_length: int

    def __init__(
        self,
        service_name: str = None,
        endpoint: str = None,
        access_token: str = None,
        max_attr_length: int = None,
    ):
        if not endpoint:
            endpoint = environ.get("OTEL_EXPORTER_JAEGER_ENDPOINT", None)
            if not endpoint:
                endpoint = splunk_env_var("TRACE_EXPORTER_URL")
                if endpoint:
                    logger.warning(
                        "%s is deprecated and will be removed soon. Please use %s instead",
                        "SPLUNK_TRACE_EXPORTER_URL",
                        "OTEL_EXPORTER_JAEGER_ENDPOINT",
                    )
            endpoint = endpoint or DEFAULT_ENDPOINT
        self.endpoint = endpoint

        if not service_name:
            service_name = splunk_env_var("SERVICE_NAME", DEFAULT_ENDPOINT)
        self.service_name = service_name

        if not access_token:
            access_token = splunk_env_var("ACCESS_TOKEN", None)
        self.access_token = access_token

        if not max_attr_length:
            try:
                max_attr_length = int(
                    splunk_env_var("MAX_ATTR_LENGTH", DEFAULT_MAX_ATTR_LENGTH)
                )
            except (TypeError, ValueError):
                logger.error("SPLUNK_MAX_ATTR_LENGTH must be a number.")
        self.max_attr_length = max_attr_length


def splunk_env_var(name: str, default: Optional[str] = None) -> Optional[str]:
    old_key = "SPLK_{0}".format(name)
    new_key = "SPLUNK_{0}".format(name)
    if old_key in environ:
        logger.warning(
            "%s is deprecated and will be removed soon. Please use %s instead",
            old_key,
            new_key,
        )
        return environ[old_key]
    return environ.get(new_key, default)



# Copyright Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys

from opentelemetry import propagate, trace
from opentelemetry.exporter.jaeger.thrift import JaegerExporter
from opentelemetry.propagators.b3 import B3Format
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from pkg_resources import iter_entry_points

from splunk_otel.excludes import excluded_instrumentations
from splunk_otel.options import Options
from splunk_otel.version import __version__

logger = logging.getLogger(__file__)
logger.setLevel(logging.INFO)

propagate.set_global_textmap(B3Format())


def start_tracing(*args, **kwargs):
    enabled = os.environ.get("OTEL_TRACE_ENABLED", True)
    if not _is_truthy(enabled):
        logger.info("tracing has been disabled with OTEL_TRACE_ENABLED=%s", enabled)
        return

    options = Options(*args, **kwargs)
    try:
        init_tracer(options)
        auto_instrument()
    except Exception:  # pylint:disable=broad-except
        sys.exit(2)


def init_tracer(options: Options):
    provider = TracerProvider(
        resource=Resource.create(
            attributes={
                "service.name": options.service_name,
                "telemetry.auto.version": __version__,
            }
        )
    )
    trace.set_tracer_provider(provider)
    exporter = new_exporter(options)
    provider.add_span_processor(BatchSpanProcessor(exporter))


def new_exporter(options: Options) -> JaegerExporter:
    exporter_options = {
        "collector_endpoint": options.endpoint,
        "max_tag_value_length": options.max_attr_length,
    }

    if options.access_token:
        exporter_options.update(
            {
                "username": "auth",
                "password": options.access_token,
            }
        )

    return JaegerExporter(**exporter_options)


def auto_instrument():
    for entry_point in iter_entry_points("opentelemetry_instrumentor"):
        if entry_point.name in excluded_instrumentations:
            logger.info(
                "%s instrumentation has been temporarily disabled by Splunk",
                entry_point.name,
            )
            continue

        try:
            entry_point.load()().instrument()  # type: ignore
            logger.debug(
                "Instrumented %s",
                entry_point.name,
            )
        except Exception:  # pylint: disable=broad-except
            logger.exception(
                "Instrumenting of %s failed",
                entry_point.name,
            )


def _is_truthy(value: any) -> bool:
    if isinstance(value, str):
        value = value.lower()
    return value in [True, 1, "true", "yes"]



#!/usr/bin/env python

# Copyright Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import os.path
import sys
from argparse import REMAINDER, ArgumentParser
from logging import getLogger
from os import environ, execl, getcwd
from shutil import which

from splunk_otel.version import format_version_info

logger = getLogger(__file__)

ap = ArgumentParser()
ap.add_argument(
    "--token",
    "-t",
    required=False,
    type=str,
    dest="token",
    help="Your SignalFx Access Token (SIGNALFX_ACCESS_TOKEN env var by default)",
)
ap.add_argument(
    "--version",
    "-v",
    required=False,
    action="store_true",
    dest="version",
    help="Print version information",
)
ap.add_argument(
    "command",
    help="Arguments for your application.",
    nargs=REMAINDER,
)


def run():
    args = ap.parse_args()
    if args.token:
        logger.warn("--token is not support yet and will have no effect.")
        os.environ["SIGNALFX_ACCESS_TOKEN"] = args.token

    site_dir = os.path.join(
        os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
        "site",
    )
    py_path = os.environ.get("PYTHONPATH", "")

    # This is being added to support applications that are being run from their
    # own executable, like Django.
    if not py_path:
        py_path = []
    else:
        py_path = py_path.split(os.path.pathsep)
    cwd_path = getcwd()
    if cwd_path not in py_path:
        py_path.insert(0, cwd_path)
    py_path = os.path.pathsep.join(py_path)

    os.environ["PYTHONPATH"] = site_dir + os.pathsep + py_path if py_path else site_dir

    if args.version:
        print(format_version_info())
        return

    if not args.command:
        ap.error(ap.format_help())

    cmd, cmd_args = args.command[0], args.command[1:]
    executable = which(cmd)
    try:
        execl(executable, cmd, *cmd_args)
    except TypeError:
        logger.error("failed to execute program: %s", " ".join(args.command))
        raise
